# configuration de base
src = "."
setwd(src)
# création des dossiers : https://stackoverflow.com/questions/53855614/r-create-a-new-folder-using-the-given-path 
dir.create(file.path(dirname(src), "data"))
dir.create(file.path(dirname(src), "out" ))
# lecture du fichier d'entrée
data = read.csv('data/le_clou.tsv', encoding="UTF-8", sep="\t", header=TRUE)
View(data)

# 1. Obtenez l'effectif de chaque catégorie grammaticale et créez un tableau avec ces données.
# création data.frame > fait seul le compte des effectifs à partir d'une colonne
pos_comptes = table(data$Etiquette)
pos_df <- as.data.frame(pos_comptes)
# nommage des colonnes de la dataFrame
colnames(pos_df) <- c("Pos", "Freq")
View(pos_df)

# 2. Représentez les effectifs obtenus pour la question 1 avec un diagramme circulaire (camembert).
# Les pourcentages pour chaque secteur du diagramme doivent également être affichés.'
# étiquettes pour le diagramme
labels = pos_df$Pos
# valeurs pour le diagramme
# slices == effectif, ou Freq. 
slices = pos_df$Freq
# pourcentages
pourcentages = round(slices/sum(slices)*100,
                     digits=2)
# nouvelles étiquettes avec pos-tag, compte et pourcentage
labels_avec_pourcent = paste(labels, slices, pourcentages)
# archive de sauvegarde 
png("out/pie-chart_le-clou.png", width=700, height=500)
# librairie pour visualisation esthétique
library(RColorBrewer)
# palette préparée avec nb de couleur égal aux nombres de catégorie (labels)
mypalette = brewer.pal(length(labels), "Set3")
pie(slices, labels=labels_avec_pourcent, col=mypalette,
    main="Effectif des catégories grammaticales",
    sub="Le loup et l'homme des frères Grimm")
# légende
legend("topright", levels(pos_df$Pos), cex=0.7, fill=mypalette)
# écriture sur fichier de sortie
dev.off()

# 3. Après calculer la longueur de chaque lemme, obtenez l'effectif de chaque longueur et créez un tableau avec ces effectifs.
# librairie pour le compte des chaines de caracteres
library(stringr)
# compte des longueurs de chaine appliqué à la colonne Mot
data$LemmeLen = str_count(data$Lemme)
longueurs = table(data$LemmeLen)
# création data.frame > fait seul le compte des effectifs à partir d'une colonne
strlen_df = as.data.frame(longueurs)
# nommage des colonnes pour visualisation
colnames(strlen_df) <- c("String_length", "Freq")
View(strlen_df)

# 4. Représentez les effectifs des longueurs de lemmes avec un diagramme en colonnes.
# préparation de l'img en sortie
png("out/barplot_le-clou.png", width=500, height=500)
# définition du diagramme
barplot(longueurs, col=c("darkblue"), main="Effectif des longueurs des mots", sub="Le clou des frères Grimm", xlab="Longueur", ylab="Effectif", ylim=c(0, 200))
dev.off()

# 5. Obtenez les effectifs des catégories grammaticales en fonction de la longueur des lemmes et remplissez un tableau comme le suivant :
# <= 4 vs > 4. 
# longueur des lemmes
# data$LemmeLen = str_count(data$Lemme)
# effectif par longueur de lemme
# lemmelen_df = as.data.frame(data$LemmeLen)
# filtre = longueur de la chaine <= 4 pour chaque lemme, avec son étiquette
pos_lemme_filtre1 = data[data$LemmeLen <=4, c("Etiquette", "LemmeLen")]
# 2nd filtre pour avoir les lemmes de < 4 caractères avec Etiquette
pos_lemme_filtre2 = data[data$LemmeLen >4, c("Etiquette", "LemmeLen")]
View(pos_lemme_filtre1)
# passage en dataframe 
lemmelen_df1 = as.data.frame(pos_lemme_filtre1)
colnames(lemmelen_df1) = c("Etiquette", "LemmeLen")
lemmelen_df2 = as.data.frame(pos_lemme_filtre2)
colnames(lemmelen_df2) = c("Etiquette", "LemmeLen")
View(lemmelen_df1)

# librairie pour distribution de fréquences 
library(plyr)
ct_lemm1 = count(lemmelen_df1, "Etiquette")
View(ct_lemm1)
ct_lemm2 = count(lemmelen_df2, "Etiquette")
View(ct_lemm2)
# merge les tables sur la colonne "Etiquette"
# https://www.datasciencemadesimple.com/joi n-in-r-merge-in-r/ 
# install.packages('dplyr')
library(dplyr)
# outer join pour que les effectifs == 0 soient inclus aussi
df = merge(x=ct_lemm1, y=ct_lemm2, by="Etiquette", all=TRUE)
colnames(df) = c("Etiquette", "<=4", ">4")
# traitement des valeurs nulles pour enlever "invalid value" 
# https://www.digitalocean.com/community/tutorials/replace-in-r
df[is.na(df)] = 0
View(df)
write.csv(df, "out/strlen_lemme.csv")
