package actors_movie;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.HashMap;
import java.nio.file.* ;
import java.nio.charset.Charset ;
import java.io.BufferedReader;
import java.io.IOException;

public class MovieCollection {
	//données membres privées pour préserver les données 
    private HashMap<String, Movie> movies;
    private HashMap<String, Keyword>keywords;
    
//    constructeur 
    public MovieCollection() {
    	movies = new HashMap<String, Movie>();
    	keywords = new HashMap<String, Keyword>();
    }
// méthode de lecture d'un fichier et de remplissage de la collection
    public void readMovieCollection(String filename) {
    	// objet Path pour la recherche du fichier (induit que filename soit le chemin relatif du document).    	
    	Path doc = Paths.get(filename);
    	Charset charset = Charset.forName("UTF-8");
    	//	gestion des exceptions Input Output avec bloc try/catch
    	try (BufferedReader reader = Files.newBufferedReader(doc, charset)) {
    		// passe la première ligne qui est le header. 
    		reader.readLine();
    		String line = reader.readLine();
    		//	lecture ligne à ligne avec boucle while.    		
	    	while (line != null) {
	    		//	traitement de la ligne en colonne
	    		String[] colonnes = line.split(";");
    			String movieTitle = colonnes[0];
    			String directorName = colonnes[1];
    			//	cast de String à int avec methode parseInt
    			int duration = Integer.parseInt(colonnes[3]);
    			int year = Integer.parseInt(colonnes[13]);
    			//	creation d'une nouvelle instance de Movie à partir des informations de la ligne
    			Movie mov = new Movie(movieTitle, directorName, duration, year);
    			//	ajout à la HashMap movies de MovieCollection du titre et de l'objet Movie :
    			movies.put(movieTitle, mov);
    			//	sélection de la dernière colonne contenant les mots-clés avec séparateur "|" :
    			String[] themes = colonnes[colonnes.length-1].split("\\|");
    			for (String word : themes) {   	
    				//	vérification de la présence du mot-clé dans les clés de la HashMap keywords de l'instance de MovieCollection:
    				if (keywords.containsKey(word)){
    					//	si mot déjà dans les clés de la HashMap, on cherche l'objet Keyword correspondant à la valeur de la clé du même nom dans keywords.
    					Keyword k = keywords.get(word);
    					// ... pour ajouter à cette instance de Keyword l'objet Movie à sa liste movieList.
    					k.addMovie(mov);
    				}  					
    				else {
    					// si mot-clé pas encore dans la HashMap : création d'une nouvelle instance de Keyword...
    					Keyword k = new Keyword(word);
    					//	... ajout de l'objet Movie au movieList de ce nouvel objet Keyword
    					k.addMovie(mov);
    					//	et enfin ajout à la HashMap keywords de MovieCollection, de la clé word avec pour valeur Keyword (contenant le Movie dans sa movieList)
    					keywords.put(word, k);
    				}
    			}
    			//	mise à jour de la variable line pour éviter la boucle infinie
	    		line = reader.readLine();
	    	}
	    	//	gestion de l'exception IO : simple affichage.
    	} catch (IOException x) {
    	System.out.println("IOException : " + x.toString()) ;
    	}
    	
    }
    //	méthode d'affichage des mots-clés avec un minimum de n occurrences dans la collection
    public void printKeywords(int n) {
    	//	extraction des objets Keywords présents en tant que valeurs de la HashMap keywords.
    	ArrayList<Keyword> keywordsSorted = new ArrayList<>(keywords.values());
    	//  tri de la ArrayList, source:
//    	https://stackoverflow.com/questions/780541/how-to-sort-a-hashmap-in-java
    	// Collection.comparing avec paramètre en méthode de classe getNbMovies + Collections.sort, source:
//		https://www.baeldung.com/java-8-comparator-comparing
    	Collections.sort(keywordsSorted, Comparator.comparingInt(Keyword::getNbMovies).reversed());
    	for (Keyword key:keywordsSorted) {
    		//	affichage des Keywords dont le paramètre getNbMovies() retourne une valeur supérieure ou égale à l'argument n.
    		if (key.getNbMovies() >= n) {
    			System.out.println(key.toString());
    		}
    	}
    }
    
    
    public void printy(int n) {
    	for (Keyword key: keywords.values()) {
    		if (n <= key.getNbMovies() ) {
    			System.out.println(key.toString());
    		}
    	}
    }
    //	méthode d'affichage des films associés au mot-clé donné en argument	
    public void printMoviesForKeyword(String keyword) {
    	if (keywords.containsKey(keyword)) {
    		//	si le mot-clé se trouve dans keywords, on cherche l'objet Keyword associé (la valeur de la clé dans la HashMap keywords)
        	Keyword w = keywords.get(keyword);
        	//	utilisation de la méthode de classe Keyword pour l'affichage des informations des objets Movie de la liste movieList de l'instance Keyword.
        	w.printMovies();
        	} else { 
        		//	affichage si le mot n'est pas reconnu/ne fait pas partie des mots-clés de la collection.
        		System.out.println("Pas de film associé à ce mot clé.");
        	}
    }
    }


