
import csv

model = "gpt"

PARAMETERS = {
    'doc' : 'scripts4training/scripts_generativeAI/gpt_out/1shot_gpt_generic',
    }

if model == 'mistral':
    PARAMETERS['MODEL'] = "mistral-small"
    # price in €
    PARAMETERS['INPUT_PRICE'] = 0.6
    PARAMETERS['OUTPUT_PRICE'] = 1.8
    PARAMETERS['MAX_INPUT'] = 32000
    from transformers import AutoTokenizer
    tokenizer = AutoTokenizer.from_pretrained("mistralai/Mixtral-8x7B-v0.1")
    def count_tok(text:str) -> int:
      """Counts actual number of token (BPE) after tokenization
      Args:
          text: document or chunk of document that will be passed in the prompt
      Returns:
          Number of tokens
      """
      return len(tokenizer.tokenize(text))

if model == 'gpt' :
    PARAMETERS['MODEL'] = "gpt-3.5-turbo-0125"
    # price in $
    PARAMETERS['INPUT_PRICE'] = 0.5
    PARAMETERS['OUTPUT_PRICE'] = 1.5
    PARAMETERS['MAX_INPUT'] = 16000
    import tiktoken
    encoding = tiktoken.encoding_for_model(PARAMETERS['MODEL'])
    def count_tok(text:str)->int:
        return len(encoding.encode(text))


TEMPLATE ="""
Put a label to the stage direction using one of the categories listed below: 
['action', 'aggression', 'aparte', 'delivery', 'entrance', 'exit', 'interaction', 'movement', 'music', 'narration', 'object', 'setting', 'toward']\n
Here are examples of stage directions with their correct label:

Hector lit.  -> action
"Il lui donne un coup dans le dos, et un autre dans l'estomac." -> aggression
Aparté.  -> aparte
Elle marmotte ici quelques mots. -> delivery
Entre un autre messager. -> entrance
Il sort avec Narbanor.  -> exit
Ils l'entraînent. -> interaction
Marche guerrière : on porte les armes de Tancrède devant lui.  -> movement
Air : du Méléagre champenois. -> music
"Ils cessent de le berner. Il se relève et prend la fuite, mais un des lutins court après lui et l'attrape à l'entrée de la coulisse, ou, pour mieux dire, on donne au lutin dans la coulisse un Arlequin empaillé qu'il emporte sur le haut d'un rocher. Pendant ce temps-là, d'autres lutins vont à Léandre et un deux lui dit :"  -> narration
Elle ferme la porte du fond et prend la clef. -> object
Au palais Strozzi -> setting
À Georgette. -> toward

This is the stage direction you have to classify: {stgdir}
You only have to give the label, no explanation or notes are needed.
Remember, you can only use one of the labels listed above."""




with open(f"{PARAMETERS['doc']}.csv", 'r', encoding='utf-8') as f:
    total_input = []
    total_output = []
    reader = csv.reader(f, delimiter=',', quotechar='"', quoting=csv.QUOTE_MINIMAL)
    for row in reader:
        input_tok = count_tok(TEMPLATE.format(stgdir=row[0]))
        print(row[0])
        output_tok = count_tok(row[2])
        print(row[2])
        total_input.append(input_tok)
        total_output.append(output_tok)
entree = sum(total_input)
print(entree)
sortie = sum(total_output)
print(sortie)
prix_entree = (entree/1000000)*PARAMETERS['INPUT_PRICE']
prix_sortie = (sortie/1000000)*PARAMETERS['OUTPUT_PRICE']
print("coût input : "+ str(prix_entree))
print("coût output : "+ str(prix_sortie))

with open(f"{PARAMETERS['doc']}_param.csv", 'a', newline='\n',  encoding='utf-8') as f:
    writer = csv.writer(f, delimiter=";", quotechar='"', quoting=csv.QUOTE_MINIMAL)
    writer.writerow(["Nb de tokens en entrée", entree])
    writer.writerow(["Prix d'entrée", prix_entree])
    writer.writerow(["Nb de tokens en sortie", sortie])
    writer.writerow(["Prix de sortie", prix_sortie])
    writer.writerow(["Prix total", prix_sortie+prix_entree])
