
from sklearn.metrics import classification_report, confusion_matrix
import csv
import os
import re
import pandas as pd
from getpass import getpass
import time
import matplotlib.pyplot as plt
from mlxtend.plotting import plot_confusion_matrix

######## configurations #######
model = 'gpt'
technique = '0shot'

PARAMETERS = {
    'TEMPERATURE' : 0.0,
    'OUTPATH' : f"scripts4training/scripts_generativeAI/{model}_out",
    'OUTDOCNAME' : f'{technique}_{model}_generic',
    "DATAPATH" : "data_train\stgdir_labelGeneric_test.csv",
    'MAX_TOK' : 20,
    'labelCol': 'labelGeneric'
    }

if model == 'mistral':
    PARAMETERS['MODEL'] = "mistral-small"
    PARAMETERS['SEED'] = 12
    PARAMETERS['MAX_INPUT'] = 32000
    # imports spécifiques au modèle
    from mistralai.client import MistralClient
    from mistralai.models.chat_completion import ChatMessage
    # PASS API KEY AS USER INPUT
    mistral_api_key = os.environ.get("MISTRAL_API_KEY")
    if not mistral_api_key :
        mistral_api_key = getpass("Mistral API Key: ")
    client = MistralClient(api_key=mistral_api_key)



if model == 'gpt' :
    PARAMETERS['MODEL'] = "gpt-3.5-turbo-0125"
    PARAMETERS['MAX_INPUT'] = 16000
    # import spécifiques au modèle
    from openai import OpenAI
    # PASS API KEY AS USER INPUT
    # openai_api_key = os.environ.get('OPENAI_API_KEY')
    # if not openai_api_key:
    openai_api_key = getpass("OpenAI API Key: ")
    client = OpenAI(api_key=openai_api_key)

######## looking for class names#############
AS13_df = pd.read_table(PARAMETERS['DATAPATH'], sep=',', dtype={'description' : 'object', 'labelGeneric': 'category', })
labelCol = PARAMETERS['labelCol']
PARAMETERS['CLASS_NAMES'] = sorted(AS13_df[labelCol].unique())
print(PARAMETERS['CLASS_NAMES'])
# label2id = {PARAMETERS['CLASS_NAMES'][i]:i for i in range(len(PARAMETERS['CLASS_NAMES']))}
# id2label = {i:PARAMETERS['CLASS_NAMES'][i] for i in range(len(PARAMETERS['CLASS_NAMES']))}
# print(id2label)


############## Ensemble des templates ###############
# pour typologie Generic
template_zero_shot = """
Put a label to the following stage direction using one of the categories listed below: 
Stage direction : {stgdir}
Categories : {cat}  \n\n
You only have to give the label, no explanation or notes are needed.
Remember, you can only use one of the labels listed above.
"""
template_few_shot = """
Put a label to the stage direction using one of the categories listed below: 
['action', 'aggression', 'aparte', 'delivery', 'entrance', 'exit', 'interaction', 'movement', 'music', 'narration', 'object', 'setting', 'toward']\n
Here are examples of stage directions with their correct label:

Hector lit.  -> action
Il cherche sous tous les meubles ->action
Il lui envoie un coup sur la tête avec ta potence.  -> aggression
"Il lui donne un coup dans le dos, et un autre dans l'estomac." -> aggression
Aparté.  -> aparte
Elle marmotte ici quelques mots. -> delivery
Éclats de rire. -> delivery
Il rentre. -> entrance
Entre un autre messager. -> entrance
Il sort avec Narbanor.  -> exit
Ariston sort. -> exit
Ils choquent leurs verres. -> interaction
Ils l'entraînent. -> interaction
Marche guerrière : on porte les armes de Tancrède devant lui.  -> movement
Il recule. -> movement
Air : De la Croisée. -> music
Air : du Méléagre champenois. -> music
"Ils cessent de le berner. Il se relève et prend la fuite, mais un des lutins court après lui et l'attrape à l'entrée de la coulisse, ou, pour mieux dire, on donne au lutin dans la coulisse un Arlequin empaillé qu'il emporte sur le haut d'un rocher. Pendant ce temps-là, d'autres lutins vont à Léandre et un deux lui dit :"  -> narration
"les prisonniers s'avancent lentement par groupes séparés, les étudiants avec les étudiants, bourgeois et marchands ensemble, le soldat seul. Les vieux semblent accablés de fatigue et de douleur. Pendant toute cette scène et les deux qui suivent, on continue d'entendre par moments les fanfares et les chants de la salle voisine."  -> narration
Tirant son flacon. -> object
Elle ferme la porte du fond et prend la clef. -> object
Au palais Strozzi -> setting
On tire le rideau. -> setting
À un officier. -> toward
À Georgette. -> toward

This is the stage direction you have to classify: {stgdir}
You only have to give the label, no explanation or notes are needed.
Remember, you can only use one of the labels listed above."""


template_one_shot = """
Put a label to the stage direction using one of the categories listed below: 
['action', 'aggression', 'aparte', 'delivery', 'entrance', 'exit', 'interaction', 'movement', 'music', 'narration', 'object', 'setting', 'toward']\n
Here are examples of stage directions with their correct label:

Hector lit.  -> action
"Il lui donne un coup dans le dos, et un autre dans l'estomac." -> aggression
Aparté.  -> aparte
Elle marmotte ici quelques mots. -> delivery
Entre un autre messager. -> entrance
Il sort avec Narbanor.  -> exit
Ils l'entraînent. -> interaction
Marche guerrière : on porte les armes de Tancrède devant lui.  -> movement
Air : du Méléagre champenois. -> music
"Ils cessent de le berner. Il se relève et prend la fuite, mais un des lutins court après lui et l'attrape à l'entrée de la coulisse, ou, pour mieux dire, on donne au lutin dans la coulisse un Arlequin empaillé qu'il emporte sur le haut d'un rocher. Pendant ce temps-là, d'autres lutins vont à Léandre et un deux lui dit :"  -> narration
Elle ferme la porte du fond et prend la clef. -> object
Au palais Strozzi -> setting
À Georgette. -> toward

This is the stage direction you have to classify: {stgdir}
You only have to give the label, no explanation or notes are needed.
Remember, you can only use one of the labels listed above."""


# décide depuis la technique en config quel template appliquer 
if technique == '0shot':
    template = template_zero_shot
if technique == 'Nshot':
    template = template_few_shot
if technique == '1shot':
    template = template_one_shot




###################



def format_prompt(stgdir:str, template: str, cat:list) :
    """    Formats the template with the document or chunk of document to form a valid prompt
    Args:
        template: config.TEMPLATE with input {document}
        chunk: portion of a document (max 32k tokens) as a list
        sections: list of custom sections from user
    Returns:
        ChatMessage: complete MistralAI ChatMessage with role "user"
    """
    if model == 'gpt':
        formatted_prompt = [{'role': 'user', 'content': template.format(stgdir=stgdir, cat=cat)}]
    if model =='mistral':
        formatted_prompt = [
            ChatMessage(role="user", content=template.format(stgdir=stgdir, cat=cat)),
            ]
    return formatted_prompt



def output(stgdir: str, prompt: str): 
    time.sleep(0.01)
    formatted_prompt = format_prompt(stgdir, prompt, PARAMETERS['CLASS_NAMES'])
    if model == 'mistral':
        response = client.chat(model=PARAMETERS['MODEL'], temperature=PARAMETERS['TEMPERATURE'], messages=formatted_prompt)
    if model == 'gpt':
        response = client.chat.completions.create(model=PARAMETERS['MODEL'], temperature=PARAMETERS['TEMPERATURE'], messages=formatted_prompt)
    answer = response.choices[0].message.content
    print("text : " + stgdir)
    print("pred : "+ answer)
    return answer

################ Fonctions pour l'ajouts des prédictions et post-processing #################
def add_pred(df):
    """
    add to df entire output as new column
    """
    df["output"] = df['description'].map(lambda stgdir : output(stgdir, template))
    print(df.info())
    return df

def write_pred(df):
    df.to_csv(f"{PARAMETERS['OUTPATH']}/{PARAMETERS['OUTDOCNAME']}.csv", index=False )


def get_1st_word(output):
    """
    clean raw output and extracts the first word (expected place of the predicted label)
    """
    output = output.split()[0]
    output = re.sub(r"\W", '', output)
    if re.match("apart[ée]", output.lower()):
        output = "aparte"
    return output.lower()

def add_clean_column(df):
    """
    add to df the 1st word of the output as the predicted label
    """
    df["labelpred_1st"] = df["output"].map(lambda output: get_1st_word(output))
    return df


def get_any(output):
    """
    extract any label from the raw output, if no match between a valid label and the output string, returns "unknown"
    """
    res = 'unknown'
    for c in PARAMETERS['CLASS_NAMES']: 
        if re.search(c.lower(), output.lower()):
            res = c
    if re.search('interaction', output.lower()):
        res = 'interaction'
    if re.search(r'apart[ée]', output.lower()):
        res = 'aparte'
    return res

def add_anymatch_column(df):
    """
    add to df labelpred_any column
    """
    df["labelpred_any"] = df["output"].map(lambda output: get_any(output))
    return df


def parameters():
    return [['model', PARAMETERS['MODEL']],['temp', PARAMETERS['TEMPERATURE']], ['max_token', PARAMETERS['MAX_TOK']]]

def comparaison():
    with open(f"{PARAMETERS['OUTPATH']}/{PARAMETERS['OUTDOCNAME']}_res.txt", 'w', newline='\n', encoding="utf-8") as csvfile:
        csvwriter = csv.writer(csvfile, delimiter=';',
                                quotechar='"', quoting=csv.QUOTE_MINIMAL)
        df = pd.read_csv(f"{PARAMETERS['OUTPATH']}/{PARAMETERS['OUTDOCNAME']}.csv", sep=',', quotechar='"', dtype={'description' : "object", PARAMETERS['labelCol'] : 'category', "output": 'category', 'labelpred_1st' : 'category', "labelpred_any": 'category'})
        y_true = df[PARAMETERS['labelCol']]
        y_pred = df['labelpred_1st']
        pred_class_names = sorted(y_pred.unique().categories.to_list())
        csvwriter.writerow(['Classification report 1st word', classification_report(y_true, y_pred) ])
        inv = [x for x in pred_class_names if x not in PARAMETERS['CLASS_NAMES']]
        csvwriter.writerow(['1e mot de l\'output hors labels valides', ";".join(inv)])
        df_comp = df[df["labelpred_1st"].isin(inv)]
        print(df_comp)
        csvwriter.writerow(['nb de stgdirection sans label valide (méthode 1st word)', df_comp.shape[0]])
        # print(classification_report(y_true, y_pred))
        y_pred_any = df["labelpred_any"]
        print(classification_report(y_true, y_pred_any))
        csvwriter.writerow(['Classification report any match in output', classification_report(y_true, y_pred_any) ])
        unknown = df[df["labelpred_any"] == 'unknown']
        csvwriter.writerow(['nb de stgdirection sans label valide (méthode any match)', unknown.shape[0]])
        print(unknown.head())


def write_param(prompt):
    with open(f"{PARAMETERS['OUTPATH']}/{PARAMETERS['OUTDOCNAME']}_param", 'w', newline='\n', encoding="utf-8") as csvfile:
        csvwriter = csv.writer(csvfile, delimiter=';',
                                quotechar='"', quoting=csv.QUOTE_MINIMAL)
        params = parameters()
        for param in params: 
            csvwriter.writerow(param)
        csvwriter.writerow(['System_prompt', prompt])


def conf_m(y_true, y_pred):
    conf_matrix = confusion_matrix(y_true, y_pred)
    fig, ax = plot_confusion_matrix(conf_mat=conf_matrix, figsize=(6, 6), cmap=plt.cm.Greens, class_names=PARAMETERS['CLASS_NAMES'])
    plt.xlabel('Predictions', fontsize=18)
    plt.ylabel('Réels', fontsize=18)
    plt.title(f'Matrice de confusion {PARAMETERS["OUTDOCNAME"]}', fontsize=18)
    plt.show()
    plt.savefig(f"../figures/{PARAMETERS['OUTDOCNAME']}.pdf", format='pdf', bbox_inches='tight')


if __name__=='__main__':
    df = pd.read_table(PARAMETERS['DATAPATH'], sep=",", dtype={'text': 'object', PARAMETERS['labelCol']: "category"})
    print(df.info())
    df_pred = add_pred(df)
    write_pred(df)
    df = add_clean_column(df)
    df = add_anymatch_column(df)
    print(df.info())
    print(df.head())
    write_pred(df)
    write_param(template)
    comparaison()
    conf_m(df[PARAMETERS['labelCol']], df['labelpred_1st'])
    